import es
import popuplib
import random
import playerlib
import gamethread
from damage import damage

info = es.AddonInfo()
info.name = 'Poker showdown script'
info.basename = 'poker'
info.version = "1.0"
info.author = "GODJonez"
info.description = "One-on-one situation can be called for a round of poker"
info.tags = "silly fun"

suits = {
    1: 'Heart  ',
    2: 'Spade  ',
    3: 'Diamond',
    4: 'Club   '}

values = {
    1: 'A ',
    2: '2 ',
    3: '3 ',
    4: '4 ',
    5: '5 ',
    6: '6 ',
    7: '7 ',
    8: '8 ',
    9: '9 ',
    10: '10',
    11: 'J ',
    12: 'Q ',
    13: 'K '}

class Card(object):
    '''
This is a card class
    '''
    def __init__(self, suit, value):
        ''' Initialize '''
        self.suit = suit
        self.value = value
    def __str__(self):
        ''' Show as string '''
        return "%s %s"%(suits[self.suit],values[self.value])
    def ispair(self, card):
        ''' Check if the cards are pairs (same value) '''
        return (self.value == card.value)
    def isflush(self, card):
        ''' Check if the cards are of same suit '''
        return (self.suit == card.suit)

class Deck(object):
    '''
This is a deck class
    '''
    def __init__(self):
        ''' Initialize '''
        self.cards = []     # this list will hold the cards
        # Now create the cards for the deck, we won't do anything with an empty one
        for suit in suits:
            for value in values:
                self.cards.append(Card(suit,value))
    def shuffle(self):
        ''' Shuffle the deck '''
        random.shuffle(self.cards)
    def take(self):
        ''' Take a card from the deck '''
        return self.cards.pop(0)
    def put(self, card):
        ''' Put a card to the deck '''
        self.cards.append(card)
    def __len__(self):
        ''' Returns the number of cards '''
        return len(self.cards)

class Hand(Deck):
    '''
This is a class representing player's hand
    '''
    def __init__(self):
        self.cards = []
        # do NOT fill with cards
    def togglediscard(self, ci):
        card = self.cards[ci]
        if card.discard:
            card.discard = False
        else:
            card.discard = True
        return card.discard
    def put(self, card):
        card.discard = False
        super(Hand, self).put(card)
    def take(self, ci):
        card = self.cards.pop(ci)
        card.discard = False
        return card
    # the rest of methods are from Deck class

    
# Create a new deck:
playdeck = Deck()
state = {
    'votes': 0,
    'players': None}

playas = {}

def load():
    ''' es_load poker '''
    p = popuplib.create("pokervote")
    
    p.editlang = "en"
    p.addline("It's one on one")
    p.addline(" ")
    p.addline("Would you like to settle")
    p.addline("the score with a round of")
    p.addline("poker instead of weapons?")
    p.addline(" ")
    p.addline("->1. Yes")
    p.addline("->2. No ")
    p.addline(" ")
    p.addline(" -- poker by GODJonez")

    p.editlang = "fi"
    p.addline("Yksi vastaan yksi")
    p.addline(" ")
    p.addline("Aseiden k\xc3\xa4yt\xc3\xb6n sijasta")
    p.addline("haluatko ratkaista tuloksen")
    p.addline("pelaamalla er\xc3\xa4n pokeria?")
    p.addline(" ")
    p.addline("->1. Kyll\xc3\xa4")
    p.addline("->2. Ei")
    p.addline(" ")
    p.addline(" -- poker by GODJonez")

    p.menuselect = _poker_vote

    es.set("poker_maxchanges",1)
    
def player_death(event_var):
    ''' event player_death '''
    tcount = es.getlivingplayercount(2)
    ctcount = es.getlivingplayercount(3)
    if tcount == 1 and ctcount == 1:
        state['players'] = playerlib.getUseridList("#alive,#human")
        if len(state['players'])>1:
            popuplib.send("pokervote", state['players'])
            state['votes'] = len(state['players'])
        
def _poker_vote(userid, choice, popupid):
    ''' menuselect of pokervote '''
    if choice == 1:
        state['votes'] = state['votes'] - 1
        if state['votes'] == 0:
            es.msg("Yes voted for poker")
            _play_poker()
    else:
        es.msg("No voted for poker")
        popuplib.unsendname("pokervote", state['players'])

def _play_poker():
    ''' start the poker play '''
    playdeck.shuffle()
    playas.clear()
    for i in state['players']:
        playas[i] = playerlib.getPlayer(i)
        playas[i].hand = Hand()
        playas[i].set("freeze", 1)
        playas[i].changes = 0
    es.server.cmd('es_fire %d planted_c4 kill'%i)
    es.server.cmd('es_fire %d hostage_entity kill'%i)
    es.server.cmd('es_fire %d func_bomb_target kill'%i)
    state['votes'] = len(state['players'])
    _deal_cards(playas)
    _generate_popups(playas)
    

def _deal_cards(x):
    ''' deal cards for all players until everyone has five '''
    for i in x:
        while len(x[i].hand) < 5:
            x[i].hand.put(playdeck.take())
    
def _generate_popups(x):
    ''' generate the popups for players to see the play '''
    for i in x:
        p = popuplib.create("pokerhand%d"%i)
        p.addline("Your hand:")
        p.addline(" ")
        for ci in range(5):
            p.addline("->%d. %s"%(ci+1, str(x[i].hand.cards[ci])))
        p.addline(" ")
        p.addline("Select cards you want to discard")
        p.addline("->9. Done")
        for ci in range(10):
            p.submenu(ci+1,p)
        p.menuselect = _poker_discard
        p.send(int(x[i]))
    popuplib.create("pokerspec")
    specs = _generate_specpopup(x)
    popuplib.find('pokerspec').send(specs)
    
def _generate_specpopup(x):
    ''' generate the popups for players to see the play '''
    maxc = es.server_var['poker_maxchanges']
    spec = popuplib.find("pokerspec")
    spec.__init__("pokerspec") # clear the popup
    spec.addline("-- Poker spectator display --")
    specs = es.getUseridList()
    for i in x:
        specs.remove(i)
        spec.addline(" ")
        spec.addline("%s: %d/%d changes"%(x[i].attributes["name"],x[i].changes,maxc))
        for ci in range(5):
            spec.addline("  %s"%str(x[i].hand.cards[ci]))
    spec.addline("-----------------------------")
    spec.addline("This display updates automatically")
    return specs

def _poker_discard(userid, choice, popupid):
    ''' menuselect of poker hand popup '''
    maxc = es.server_var['poker_maxchanges']
    p = popuplib.find("pokerhand%d"%userid)
    if playas[userid].changes < maxc:
        if choice-1 in range(5):
            cstate = playas[userid].hand.togglediscard(choice-1)
            if cstate:
                line = "%d. %s"%(choice, str(playas[userid].hand.cards[choice-1]))
            else:
                line = "->%d. %s"%(choice, str(playas[userid].hand.cards[choice-1]))
            p.modline(2+choice,line)
        elif choice == 9:
            playas[userid].changes = playas[userid].changes + 1
            _discard_cards(userid)
            if playas[userid].changes == maxc:
                state['votes'] = state['votes'] - 1
                if state['votes'] == 0:
                    _poker_finals()
                else:
                    p.modline(9, "Waiting for other players...")
                    p.modline(10, " ")
        
def _discard_cards(userid):
    ''' executed when the player chooses 9 and is allowed to change cards'''
    hand = playas[userid].hand
    maxc = 5
    ci = 0
    while ci < maxc:
        card = hand.cards[ci]
        if card.discard:
            playdeck.put(hand.take(ci))
            hand.put(playdeck.take())
            maxc = maxc - 1
        else:
            ci = ci + 1
    p = popuplib.find("pokerhand%d"%userid)
    for ci in range(5):
        p.modline(ci+3,"->%d. %s"%(ci+1,str(hand.cards[ci])))
    specs = _generate_specpopup(playas)
    popuplib.find('pokerspec').update(specs)

def _poker_finals():
    ''' end of play, check results '''
    winner = 0
    highscore = 0
    subscore = 0
    for i in playas:
        thighscore, tsubscore = _count_scores(playas[i].hand.cards)
        if thighscore > highscore:
            winner = i
            highscore = thighscore
            subscore = tsubscore
        elif thighscore == highscore and tsubscore > subscore:
            winner = i
            subscore = tsubscore
        pp = popuplib.active(i)['object']
        pp.unsend(i)
        pp.delete()
    specs = _generate_specpopup(playas)
    ps = popuplib.find('pokerspec')
    ps.modline(1, '-- FINAL RESULTS --')
    ps.modline(17, "0. Close")
    ps.update(specs)
    gamethread.delayed(0.2, ps.send, playas)
    es.msg('#green','%s won this round of poker!'%playas[winner].attributes['name'])
    gamethread.delayed(3, _winner_killer, winner)
    playdeck.__init__() # redo the deck

def _winner_killer(winner):
    for i in playas:
        if i != winner:
            damage.damage(i, 10000, 32, winner)

def _count_scores(cards):
    ''' counts relative score value for player's cards '''
    cards.sort(lambda a,b: cmp(a.value, b.value))
    # test for different cases
    flush = _isflush(cards)
    straight = _countstraight(cards)
    commons = _countcommons(cards)
    if flush and straight and cards[0].value == 1 and cards[1].value == 10:
        # Royal flush
        score = 10
        subscore = straight * 4 + (4 - cards[0].suit)
        return (score, subscore)
    if commons[0] == 4:
        # 4 same
        score = 9
        subscore = commons[1]
        return (score, subscore)
    if flush and straight:
        # Straight flush
        score = 8
        subscore = straight * 4 + (4 - cards[0].suit)
        return (score, subscore)
    if commons[0] == 3.5:
        # Full house
        score = 7
        subscore = 13*commons[1] + commons[2]
        return (score, subscore)
    if flush:
        # Flush
        score = 6
        subscore = cards[4].value * 4 + (4 - cards[0].suit)
        return (score, subscore)
    if straight:
        # Straight
        score = 5
        subscore = straight * 4 + (4 - cards[4].suit)
        return (score, subscore)
    if commons[0] == 3:
        # Three same
        score = 4
        subscore = commons[1]
        return (score, subscore)
    if commons[0] == 2:
        pairs = 0
        for ci in range(4):
            if cards[ci].ispair(cards[ci+1]):
                pairs = pairs + 1
        if pairs == 2:
            # Two pairs
            score = 3
            subscore = commons[1]
            return (score, subscore)
    # everything else fails...
    score = 2
    subscore = 0
    maxval = 0
    for card in cards:
        subscore = subscore + card.value
        if card.value > maxval:
            maxval = card.value
        if card.value == 1:
            maxval = 14
    subscore = subscore + maxval*100
    
    return (score, subscore)

def _isflush(cards):
    ''' checks if all the cards are of the same suit '''
    for ci in range(1,len(cards)):
        if not cards[ci].isflush(cards[0]):
            return False
    return True

def _countstraight(cards):
    startvalue = cards[0].value
    startscore = startvalue
    if startvalue == 1:
        if cards[1].value == 10:
            startscore = 10
            startvalue = 9
    for ci in range(1,len(cards)):
        startvalue = startvalue + 1
        if cards[ci].value != startvalue:
            return 0
    return startscore

def _countcommons(cards):
    commons = 0
    maxcommons = 0
    biggest = 0
    value = 0
    for card in cards:
        testvalue = card.value
        if testvalue==1:testvalue=14
        
        if value != testvalue:
            if commons > maxcommons:
                maxcommons = commons
                biggest = value
            elif commons == maxcommons and value > biggest:
                biggest = value
            value = testvalue
            commons = 1
        else:
            commons = commons + 1
    if (commons == 2 and maxcommons == 3) or (commons == 3 and maxcommons == 2):
        # full house
        if value > biggest:
            return (3.5, value, biggest)
        else:
            return (3.5, biggest, value)
    if commons > maxcommons:
        maxcommons = commons
        biggest = value
    elif commons == maxcommons and value > biggest:
        biggest = value
    return (commons, biggest)
